//-----------------------------------------------------------------------------
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
//-----------------------------------------------------------------------------
#define STRING_LENGTH 256
#define TEXT_SPEC_LENGTH 81
#define MAXIMUM_COMMANDS 100

typedef char EditStringType[2 * TEXT_SPEC_LENGTH];

typedef char String[STRING_LENGTH];

typedef enum {none,text,line_numbers} LineRangeStyleType;

typedef int PairOfIntsType[2];

typedef union {
    EditStringType Text;
    PairOfIntsType LineNumbers;
} LineRangeDataType;

typedef struct {
    LineRangeStyleType LRStyle;
    LineRangeDataType LRData;
    char EditCommand;
    EditStringType EditData;
} CommandType;

typedef CommandType CommandArrayType[MAXIMUM_COMMANDS];
//-----------------------------------------------------------------------------
void Chomp(char * String) {

    if (String[strlen(String)-1] == '\n') {
        String[strlen(String)-1] = '\0';
    }
}
//-----------------------------------------------------------------------------
void PrintCommandDetails(CommandType Command) {

    switch (Command.LRStyle) {
        case none:
            printf("All lines\n");
            break;
        case text:
            printf("Lines containing \"%s\"\n",Command.LRData.Text);
            break;
        case line_numbers:
            printf("Lines %d to %d\n",Command.LRData.LineNumbers[0],
Command.LRData.LineNumbers[1]);
            break;
        default:
            printf("ERROR: Unknown LRStyle\n");
            break;
    }
    printf("Edit command : %c\n",Command.EditCommand);
    printf("Edit data    : %s\n",Command.EditData);
}
//-----------------------------------------------------------------------------
int LoadEditCommands(char *FileName,CommandArrayType Commands) {

    FILE *CommandFile;
    String InputLine;
    int CommandIndex;
    char *Start,*End;

    if ((CommandFile = fopen(FileName,"r")) == NULL) {
        perror("Failed to open command file");
        return(0);
    }

    CommandIndex = 0;
//----Read a line off the command file
    while (fgets(InputLine,STRING_LENGTH,CommandFile) != NULL) {
//----Remove trailing \n
        Chomp(InputLine);
        Start = InputLine;
//----If a delimiter
        if (*Start == '/') {
            Commands[CommandIndex].LRStyle = text;
            Start++;
            End = strchr(Start,'/');
            *End = '\0';
            strcpy(Commands[CommandIndex].LRData.Text,Start);
            Start = End+1;
//----If a line number
        } else if (isdigit(*Start)) {
            Commands[CommandIndex].LRStyle = line_numbers;
//----Find comma and end string, then take line number
            End = strchr(Start,',');
            *End = '\0';
            Commands[CommandIndex].LRData.LineNumbers[0] = atoi(Start);
            Start = End+1;
//----Find slash and end string, then take line number
            End = strchr(Start,'/');
            *End = '\0';
            Commands[CommandIndex].LRData.LineNumbers[1] = atoi(Start);
            Start = End+1;
//----If a command character
        } else if (isalpha(*Start)) {
            Commands[CommandIndex].LRStyle = none;
        }
//----Extract command
        Commands[CommandIndex].EditCommand = *Start;
        Start++;
//----Save the rest
        strcpy(Commands[CommandIndex].EditData,Start);

#ifdef DEBUG
        PrintCommandDetails(Commands[CommandIndex]);
#endif

//----Move on to next array position
        CommandIndex++;
    }

#ifdef DEBUG
    printf("================================================\n");
#endif

    fclose(CommandFile);
    return(CommandIndex);
}
//-----------------------------------------------------------------------------
int LineInRange(String InputLine,int CurrentLineNumber,CommandType Command) {

    switch (Command.LRStyle) {
        case none:
            return(1);
            break;
        case text:
            return(strstr(InputLine,Command.LRData.Text) != NULL);
            break;
        case line_numbers:
            return(Command.LRData.LineNumbers[0] <= CurrentLineNumber &&
CurrentLineNumber <= Command.LRData.LineNumbers[1]);
            break;
        default:
            return(0);
            break;
    }
}
//-----------------------------------------------------------------------------
int DoEdit(String InputLine,CommandType Command) {

    String TempLine;
    String Delimiter,CopyEditData;
    char *SearchString,*ReplaceString,*StringPosition;

    switch (Command.EditCommand) {
        case 'A':
            strcat(InputLine,Command.EditData);
            break;
        case 'I':
            strcpy(TempLine,Command.EditData);
            strcat(TempLine,InputLine);
            strcpy(InputLine,TempLine);
            break;
        case 'O':
            printf("%s\n",Command.EditData);
            break;
        case 'd':
            return(1);
            break;
        case 's':
            Delimiter[0] = Command.EditData[0];
            Delimiter[1] = '\0';
            strcpy(CopyEditData,&Command.EditData[1]);
            SearchString = strtok(CopyEditData,Delimiter);
            ReplaceString = strtok(NULL,Delimiter);
            while ((StringPosition = strstr(InputLine,SearchString)) != NULL) {
                strncpy(TempLine,InputLine,StringPosition-InputLine);
                TempLine[StringPosition-InputLine] = '\0';
                strcat(TempLine,ReplaceString);
                strcat(TempLine,&InputLine[(StringPosition-InputLine)+
strlen(SearchString)]);
                strcpy(InputLine,TempLine);
            }
            break;
        default:
            printf("ERROR: Unknown command %c\n",Command.EditCommand);
            PrintCommandDetails(Command);
            break;
    }

    return(0);
}
//-----------------------------------------------------------------------------
int main(int argc,char *argv[]) {

    CommandArrayType Commands;
    int NumberOfCommands,CommandIndex;
    int CurrentLineNumber;
    String InputLine;
    int LineDeleted;

    if (argc != 2) {
        printf("Usage: %s <command file>\n",argv[0]);
        exit(-1);
    }

    if ((NumberOfCommands = LoadEditCommands(argv[1],Commands)) > 0) {
        CurrentLineNumber = 0;
        while (fgets(InputLine,STRING_LENGTH,stdin) != NULL) {
            Chomp(InputLine);
            CurrentLineNumber++;
            LineDeleted = 0;
#ifdef DEBUG
            printf("================================================\n");
            printf("Line %3d: %s\n",CurrentLineNumber,InputLine);
#endif
            for (CommandIndex = 0;CommandIndex<NumberOfCommands && !LineDeleted;
CommandIndex++) {
#ifdef DEBUG
                printf("Edit command %d\n",CommandIndex+1);
                PrintCommandDetails(Commands[CommandIndex]);
#endif
                if (LineInRange(InputLine,CurrentLineNumber,
Commands[CommandIndex])) {
#ifdef DEBUG
                    printf("Edit command applies\n");
#endif
                    LineDeleted = DoEdit(InputLine,Commands[CommandIndex]);
#ifdef DEBUG
                    printf("Updated line: %s\n",InputLine);
#endif
                } else {
#ifdef DEBUG
                    printf("Edit command does not apply\n");
#endif
                }
            }
            if (!LineDeleted) {
                printf("%s\n",InputLine);
            }
        }
    }

    return(EXIT_SUCCESS);
}
//-----------------------------------------------------------------------------
