//-----------------------------------------------------------------------------
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <pthread.h>
#include <unistd.h>
#include <sys/wait.h>
#include <sys/time.h>
#include <sys/resource.h>
#include <signal.h>

#define MY_FIFO "FIBOPIPE"
#define TRUE 1
#define FALSE 0
//-----------------------------------------------------------------------------
//----Function to compute the Nth Fibonacci number recursively
long Fibonacci(long WhichNumber) {

//----If the 1st or second number, return the answer directly
    if (WhichNumber <= 1) {
        return(WhichNumber);
//----Otherwise return the sum of the previous two Fibonacci numbers
    } else {
        return(Fibonacci(WhichNumber - 2) + Fibonacci(WhichNumber - 1));
    }
}
//-----------------------------------------------------------------------------
void * FibonacciThread(void * Data) {

    int WhichNumber;
    extern int NumberOfThreads;

    WhichNumber = *((int *)Data);
    printf("S: Fibonacci %d is %ld\n",WhichNumber,Fibonacci(WhichNumber));
    
    NumberOfThreads--;
    return(NULL);
}
//-----------------------------------------------------------------------------
void SIGXCPUHandler(int TheSignal) {

    extern int ChildPID;

    if (TheSignal != SIGXCPU) {
        printf("S: That's weird, I got a signal other than SIGXCPU\n");
    }
    printf("S: Received a SIGXCPU, ignoring any more\n");
    if (signal(SIGXCPU,SIG_IGN) == SIG_ERR) {
        perror("Could not ignore SIGXCPU");
        exit(EXIT_FAILURE);
    }   
    printf("S: Received a SIGXCPU, sending SIGUSR1 to interface\n");
    if (kill(ChildPID,SIGUSR1) == -1) {
        perror("Could not send SIGUSR1 to interface");
        exit(EXIT_FAILURE);
    }
}
//-----------------------------------------------------------------------------
void SetCPUTimeLimit(int CPUTimeLimit) {

    struct rlimit ResourceLimits;

    printf("S: Setting CPU limit to %ds\n",CPUTimeLimit);
//----Limit the CPU time. Need to get old one for hard limit field
    if (getrlimit(RLIMIT_CPU,&ResourceLimits) == -1) {
        perror("CPULimitedRun - Getting resource limit:");
        exit(EXIT_FAILURE);
    }
//----Set new CPU limit in ms (sent in secs)
    ResourceLimits.rlim_cur = CPUTimeLimit; 
    if (setrlimit(RLIMIT_CPU,&ResourceLimits) == -1) {
        perror("CPULimitedRun - Setting resource limit:");
        exit(EXIT_FAILURE);
    }
}
//-----------------------------------------------------------------------------
void CreatePipe(void) {

    unlink(MY_FIFO);
    if (mkfifo(MY_FIFO,0700) == -1) {
        perror("Making FIFO");
        exit(EXIT_FAILURE);
    }
}
//-----------------------------------------------------------------------------
void DeletePipe(void) {

    unlink(MY_FIFO);
}
//-----------------------------------------------------------------------------
void ReportCPU(void) {

    struct rusage Usage;
    long TotalSec,TotalUsec;

    getrusage(RUSAGE_SELF,&Usage);
    TotalSec = Usage.ru_utime.tv_sec + Usage.ru_stime.tv_sec + 
(Usage.ru_utime.tv_usec + Usage.ru_stime.tv_usec)/1000000;
    TotalUsec = (Usage.ru_utime.tv_usec + Usage.ru_stime.tv_usec) % 1000000;
    printf("S: Server has used %lds %ldus\n",TotalSec,TotalUsec);
}
//-----------------------------------------------------------------------------
void DealWithRequests(FILE * FIFORead) {

    int FibonacciNumber;
    pthread_t NewThread;
    extern int NumberOfThreads;

    while (fscanf(FIFORead," %d",&FibonacciNumber) == 1 && 
FibonacciNumber > 0) {
        ReportCPU();
        printf("S: Received %d from interface\n",FibonacciNumber);
        if (pthread_create(&NewThread,NULL,FibonacciThread,&FibonacciNumber) 
!= 0) {
            perror("Creating thread");
            exit(EXIT_FAILURE);
        }
        NumberOfThreads++;
        if (pthread_detach(NewThread) != 0) {
            perror("Detaching thread");
            exit(EXIT_FAILURE);
        }
        printf("S: Created and detached the thread for %d\n",FibonacciNumber);
    }
}
//-----------------------------------------------------------------------------
void WaitForThreads(void) {

    extern int NumberOfThreads;

    while (NumberOfThreads > 0) {
        printf("S: Waiting for %d threads\n",NumberOfThreads);
        sleep(1);
    }
}
//-----------------------------------------------------------------------------
void WaitForChild(int ChildPID) {

    int DeadPID;
    int DeadStatus;

    DeadPID = waitpid(ChildPID,&DeadStatus,0);
    printf("S: Child %d completed with status %d\n",DeadPID,DeadStatus);
}
//-----------------------------------------------------------------------------
int main(int argc,char *argv[]) {

    FILE *FIFORead;
    extern int ChildPID;
    extern int NumberOfThreads;

    CreatePipe();

    if ((ChildPID = fork()) == -1) {
        perror("Could not fork");
        exit(EXIT_FAILURE);
    }
 
    if (ChildPID == 0) {
        execlp("ThreadFibInterface.out","ThreadFibInterface",MY_FIFO,NULL);
        perror("ERROR: Could not exec ThreadFibInterface:");
        exit(EXIT_FAILURE);
    } else {
        if (signal(SIGXCPU,SIGXCPUHandler) == SIG_ERR) {
            perror("Could not install SIGXCPU handler");
            exit(EXIT_FAILURE);
        }   
//----Do not restart interrupted system calls
        if (siginterrupt(SIGXCPU,TRUE) == -1) {
            perror("Abandoning interrupted system calls");
            exit(EXIT_FAILURE);
        }   
        SetCPUTimeLimit(atoi(argv[1]));
        if ((FIFORead = fopen(MY_FIFO,"r")) == NULL) {
            perror("Opening FIFO to read");
            exit(EXIT_FAILURE);
        }
        NumberOfThreads = 0;
        DealWithRequests(FIFORead);
        fclose(FIFORead);
        WaitForThreads();
        ReportCPU();
        WaitForChild(ChildPID);
    }

    DeletePipe();

    return(EXIT_SUCCESS);
}
//-----------------------------------------------------------------------------
int NumberOfThreads;
int ChildPID;
//-----------------------------------------------------------------------------
